from window import win

def s(e):
    funcs[0](2)
def di(e):
    funcs[1]()
def ac(e):
    funcs[2]()
def show(val):
    u.itemconfig(text,text=f'选择的值为：{val}')

a=win()
u=a.u
u.add_button((2,5),'选定第三个选项',command=s)
u.add_button((180,5),'禁用',command=di)
u.add_button((250,5),'激活',command=ac)
_,_,_,funcs,_=u.add_scalebar((100,50),direction='y',width=200,data=range(101),command=show)
text=u.create_text((5,80),text='选择的值为：None',font='微软雅黑')

a.go()



from .TinUI import BasicTinUI, TinUI, TinUIXml, TinUITheme, ExpandPanel, VerticalPanel, HorizonPanel
from .TinUIDialog import show_msg, show_info, show_success, show_warning, show_error, show_question,\
    ask_string, ask_integer, ask_float, ask_choice

from . import extension


from window import *
from tkinter import Tk

def get1(event):
    lb1=x.tags['lb1'][0]
    entry=x.tags['en'][0]
    print(lb1,entry.get())
    u.itemconfig(lb1,text=entry.get())

xml='''
<tinui>
<line x='5' y='44' anchor='nw' lineanchor='w' pady='20'>
    <button text="获取输入值" font='宋体 16' command='self.funcs["get1"]'></button>
    <entry width='200'>en</entry>
    <line anchor='nw' padx='20'>
        <button text="获取输入值" font='宋体 16' command='self.funcs["get1"]'></button>
        <entry width='200'></entry>
    </line>
    <line anchor='nw' y='120'>
        <button text="获取输入值" font='宋体 16' command='self.funcs["get1"]'></button>
        <entry width='200'></entry>
    </line>
</line>
<line>
    <button text="hi, this is a xml TinUI test" command='print'></button>
</line>
<line x='100'>
    <label text='&lt;输入内容显示]' outline=''>lb1</label>
</line>
</tinui>
'''

a=Tk()



u=TinUI(a)
x=TinUIXml(u)
x.environment(globals())
x.loadxml(xml)
u.pack(fill='both',expand=True)
a.mainloop()



from window import win

def run(event):
    from time import sleep
    for button in (b1,ba,bb,bc,bd):
        button[2][1]()
    for i in range(1,101):
        sleep(0.02)
        goto(i)
    for button in (b1,ba,bb,bc,bd):
        button[2][2]()
def to60(event):
    goto(60)
def stop(event):
    funcs[1]()
def error(event):
    funcs[2]()
def normal(event):
    funcs[0]()

a=win()
u=a.u
b1=u.add_button((2,5),'全过程',command=run)
ba=u.add_button((80,5),'固定到60%',command=to60)
bb=u.add_button((180,5),'暂停',command=stop)
bc=u.add_button((240,5),'意外错误',command=error)
bd=u.add_button((340,5),'恢复正常',command=normal)
_,_,_,goto,funcs,_=u.add_progressbar((2,50))
a.go()



"""
TinUI拓展功能包
"""

Extension='TinUI'


def buttonlize(tinui,uid,bg='#fbfbfb',line='#CCCCCC',activebg='#f5f5f5',activeline='#e5e5e5',command=None):#按钮化tinui控件
    def in_button(e):
        tinui.itemconfig(outline,outline=activeline,fill=activeline)
    def out_button(e):
        tinui.itemconfig(back,fill=bg,outline=bg)
        tinui.itemconfig(outline,outline=line,fill=line)
    def on_click(e):
        tinui.itemconfig(back,fill=activebg,outline=activebg)
        tinui.after(500,lambda : out_button(None))
        if command:
            command()

    x1,y1,x2,y2=tinui.bbox(uid)
    x1+=1
    y1+=1
    x2-=1
    y2-=1
    outlinepos=(x1,y1,x2,y1,x2,y2,x1,y2)
    nameid=uid+'-buttonlize'
    outline=tinui.create_polygon(outlinepos,width=9,fill=line,outline=line,tags=nameid)
    back=tinui.create_polygon(outlinepos,width=7,fill=bg,outline=bg,tags=nameid)
    tinui.lower(nameid,uid)

    tinui.tag_bind(nameid,'<Button-1>',on_click)
    tinui.tag_bind(nameid,'<Enter>',in_button)
    tinui.tag_bind(nameid,'<Leave>',out_button)
    tinui.tag_bind(uid,'<Button-1>',on_click)
    tinui.tag_bind(uid,'<Enter>',in_button)
    tinui.tag_bind(uid,'<Leave>',out_button)



from window import win

def close(e):
    func.close_all()

def expand(e):
    func.open_all()

a=win()
u=a.u
u.add_button((10,5),'全部关闭',command=close)
u.add_button((100,5),'全部展开',command=expand)
func=u.add_treeview((5, 40))[-2]

a.go()



import sys
sys.path.append('..')
from TinUI import *
#import extension
from tkinter import Tk


class TinUIWindow(Tk):

    def __init__(self,title='tinui-window',width=500,height=500,tran='#01ff11'):
        super().__init__()
        self.title=title
        self.tran=tran
        self.config(width=width,height=height,background=tran)
        self.attributes('-transparent',self.tran)
        self.__load_tinui()

    def __load_tinui(self):
        self.titlebar=BasicTinUI(self,height=40,bg=self.tran,relief='flat')
        self.titlebar.place(x=0,y=0,relwidth=1)
        width=self.cget('width')
        height=self.cget('height')
        self.titlebar.create_polygon((12,12,width-12,12,width-12,50,12,50,12,12),
                                     width=15,fill='#f1f3f9',outline='#f1f3f9')
        bbox=self.titlebar.bbox('all')
        self.titlebar.add_button((width-25,15),text='×',anchor='nw',
                                 bg='#f1f3f9',line='#f1f3f9',fg='black',
                                 activebg='#c42b1c',activeline='#c42b1c',)
        width=bbox[2]-bbox[0]
        self.main=BasicTinUI(self,bg='#f1f3f9')
        self.main.place(x=bbox[0]+2,y=40,width=width-3,height=height-46)
        self.xml=TinUIXml(self.main)


class win:
    def __init__(self):
        self.r=Tk()
        self.r.geometry('500x500+10+10')
        self.u=TinUI(self.r)#,bg='#01FF11')
        self.u.pack(fill='both',expand=True)
        self.uixml=TinUIXml(self.u)
    def go(self):
        self.r.mainloop()


if __name__=='__main__':
    #root=win()
    #root.go()
    r=TinUIWindow('test')
    r.mainloop()



from window import win

def getin(*e):
    con=func.get()
    u.itemconfig(p,text='输入内容：'+con)

def renormal(*e):
    func.normal()

def error(*e):
    func.error()

def disable(*e):
    func.disable()

def active(*e):
    func.active()

def insert(*e):
    func.insert(text='insert text')


a=win()
u=a.u
u.add_button((10,5),'获取输入内容',command=getin)
u.add_button((150,5),'禁用输入框',command=disable)
u.add_button((250,5),'恢复输入框',command=renormal)
u.add_button((350,5),'显示错误样式',command=error)
entry,func,_=u.add_entry((180,50),anchor='n',width=300)
p=u.add_paragraph((5,100),text='输入内容：')

u.add_button((10,200),'用active恢复',command=active)
u.add_button((10,250),'插入文本',command=insert)

a.go()



from typing import Union
from tinui import TinUI, BasicTinUI, TinUITheme

class TinUIDark(TinUITheme):
    '''
    这是TinUI黑暗模式的托管类，不是继承类
    调用时这需要注意文本和数据，样式类参数无需理会
    '''

    def __init__(self,ui:Union[TinUI,BasicTinUI]):
        super().__init__(ui,'tinui-dark-theme')
        self.label='dark'
        self.ui['background']='#202020'
    
    def add_title(self,pos,*arg,**kw):
        return self.ui.add_title(pos,*arg,**{
            **{'fg':'#ffffff'},
            **kw})
    
    def add_paragraph(self,pos,*arg,**kw):
        return self.ui.add_paragraph(pos,*arg,**{
            **{'fg':'#ffffff'},
            **kw})

    def add_button(self,pos,*arg,**kw):
        return self.ui.add_button(pos,*arg,**{
            **{
                'fg':'#ffffff','bg':'#2d2d2d',
                'activefg':'#e8e8e8','activebg':'#323232',
                'line':'#303030','linew':1,'activeline':'#303030',
            },**kw})

    def add_checkbutton(self,pos,*arg,**kw):
        return self.ui.add_checkbutton(pos,*arg,**{
            **{
                'fontfg':'#ffffff','fg':'#989898','bg':'#1d1d1d',
                'activefg':'#9E9E9E','activebg':'#2a2a2a',
                'onfg':'black','onbg':'#4CC2FF',
            },**kw})

    def add_label(self,pos,*arg,**kw):
        return self.ui.add_label(pos,*arg,**{
            **{
                'fg':'#ffffff','bg':'#202020',
                'outline':'#202020',
            },**kw})

    def add_entry(self,pos,*arg,**kw):
        return self.ui.add_entry(pos,*arg,**{
            **{
                'fg':'#ffffff','bg':'#2d2d2d',
                'activefg':'#ffffff','activebg':'#323232',
                'onfg':'#ffffff','onbg':'#1f1f1f',
                'line':'#303030','activeline':'#303030',
                'outline':'#9a9a9a','onoutline':'#4CC2FF','insert':'#e0e0e0',
            },**kw})

    def add_separate(self,pos,*arg,**kw):
        return self.ui.add_separate(pos,*arg,**{
            **{
                'fg':'#323232',
            },**kw})

    def add_radiobutton(self,pos,*arg,**kw):
        return self.ui.add_radiobutton(pos,*arg,**{
            **{
                'fg':'#f2f2f2','bg':'#2b2b2b',
                'activefg':'#ffffff','activebg':'#373737',
            },**kw})

    def add_link(self,pos,*arg,**kw):
        return self.ui.add_link(pos,*arg,**{
            **{
                'fg':'#99EBFF',
                'activefg':'#99EBFF','activebg':'#2d2d2d',
            },**kw})

    def add_waitbar1(self,pos,*arg,**kw):
        # return self.ui.add_waitbar1(pos,*arg,**{
        #     **{
        #         'fg':'#b2b8f2','bg':'#202020','okfg':'#b2b8f2','bd':5,
        #     },**kw})
        ...

    def add_labelframe(self,uids:tuple,*arg,**kw):
        return self.ui.add_labelframe(uids,*arg,**{
            **{
                'fg':'#1D1D1D','bg':'#272727',
            },**kw})

    def add_waitbar2(self,pos,*arg,**kw):
        # return self.ui.add_waitbar2(pos,*arg,**{
        #     **{
        #         'fg':'#b2b8f2','bg':'#202020','okcolor':'#6ccb5f',
        #     },**kw})
        ...

    def add_combobox(self,pos,*arg,**kw):
        return self.ui.add_combobox(pos,*arg,**{
            **{
                'fg':'#cfcfcf','bg':'#2d2d2d','outline':'#303030',
                'activefg':'#ffffff','activebg':'#323232','activeline':'#303030',
                'onfg':'#cecece','onbg':'#272727','online':'#303030',
                'listfg':'#ffffff','listactivefg':'#ffffff','listactivebg':'#383838',
                'listonfg':'#D1D1D1','listonbg':'#343434','listsel':'#383838',
                'scrollbg':'#2c2c2c','scrollcolor':'#9f9f9f','scrollon':'#a0a0a0',
            },**kw})

    def add_progressbar(self,pos,*arg,**kw):
        return self.ui.add_progressbar(pos,*arg,**{
            **{
                'fg':'#9a9a9a','bg':'#4CC2FF','back':'#202020',
                'fontc':'#0000ff',
            },**kw})

    def add_table(self,pos,*arg,**kw):
        return self.ui.add_table(pos,*arg,**{
            **{
                'outline':'#9a9a9a','fg':'#ffffff','bg':'#2d2d2d',
                'headbg':'#202020',
            },**kw})

    def add_onoff(self,pos,*arg,**kw):
        return self.ui.add_onoff(pos,*arg,**{
            **{
                'fg':'#cccccc','bg':'#1d1d1d',
                'activefg':'#cfcfcf','activebg':'#2a2a2a',
                'onactivefg':'#000000','onactivebg':'#47B1E8',
                'onfg':'#000000','onbg':'#4CC2FF',
            },**kw})

    def add_spinbox(self,pos,*arg,**kw):
        return self.ui.add_spinbox(pos,**{
            **{
              'fg':'#ffffff','bg':'#2d2d2d','line':'#303030',
              'activefg':'#c0c0c0','activebg':'#323232',
              'boxfg':'#cfcfcf','boxbg':'#2c2c2c',
              'boxactivefg':'#d2d2d2','boxactivebg':'#383838',
              'onfg':'#9f9f9f','onbg':'#343434',
            },**kw})

    def add_image(self,pos,*arg,**kw):
        return self.ui.add_image(pos,*arg,**kw)

    def add_scalebar(self,pos,*arg,**kw):
        return self.ui.add_scalebar(pos,*arg,**{
            **{
                'fg':'#4CC2FF','bg':'#9a9a9a','activefg':'#4BB5EC',
                'buttonbg':'#454545','buttonoutline':'#353535',
            },**kw})

    def add_info(self,pos,*arg,**kw):
        # return self.ui.add_info(pos,*arg,**{
        #     **{
        #         'fg':'#b2b8f2','bg':'#2d2d2d','info_fg':'#ffffff',
        #     },**kw})
        ...

    def add_menubar(self,uid,*arg,**kw):
        return self.ui.add_menubar(uid,*arg,**{
            **{
                'fg':'#ffffff','bg':'#2c2c2c','line':'#3d3d3d',
                'activefg':'#ffffff','activebg':'#383838','activeline':'#383838',
                'onfg':'#ffffff','onbg':'#343434','online':'#343434',
            },**kw})

    def add_tooltip(self,uid,*arg,**kw):
        return self.ui.add_tooltip(uid,*arg,**{
            **{
                'fg':'#e8e8e8','bg':'#353535',
            },**kw})

    def add_waitbar3(self,pos,*arg,**kw):
        return self.ui.add_waitbar3(pos,*arg,**{
            **{
                'fg':'#4CC2FF','bg':'#202020','okcolor':'#4CC2FF',
            },**kw})

    def add_textbox(self,pos,*arg,**kw):
        return self.ui.add_textbox(pos,*arg,**{
            **{
                'fg':'#ffffff','bg':'#2d2d2d',
                'outline':'#9a9a9a','onoutline':'#4CC2FF',
                'scrollbg':'#2e2e2e','scrollcolor':'#9f9f9f','scrollon':'#a0a0a0',
            },**kw})

    def add_scrollbar(self,pos,widget,*arg,**kw):
        return self.ui.add_scrollbar(pos,widget,*arg,**{
            **{
                'bg':'#2e2e2e','color':'#9f9f9f','oncolor':'#a0a0a0',
            },**kw})

    def add_listbox(self,pos,*arg,**kw):
        return self.ui.add_listbox(pos,*arg,**{
            **{
                'bg':'#2b2b2b','fg':'white',
                'activefg':'#ffffff','activebg':'#373737',
                'onfg':'#ffffff','onbg':'#333333','sel':'#115990',
                'scrollbg':'#2e2e2e','scrollcolor':'#9f9f9f','scrollon':'#a0a0a0',
            },**kw})

    # def add_canvas(self,pos,*arg,**kw):
    #     return self.ui.add_canvas(pos,*arg,**{
    #         **{
    #             'outline':'#808080','linew':1,
    #             'scrollbg':'#2e2e2e','scrollcolor':'#9f9f9f','scrollon':'#a0a0a0',
    #         },**kw})
    
    def add_pipspager(self,pos,*arg,**kw):
        return self.ui.add_pipspager(pos,*arg,**{
            **{
                'bg':'#202020','fg':'#9a9a9a',
                'buttonfg':'#9f9f9f','buttonbg':'#2c2c2c',
                'activefg':'#cfcfcf','activebg':'#2c2c2c',
                'buttononfg':'#cfcfcf','buttononbg':'#2c2c2c',
            },**kw})

    def add_notebook(self,pos,*arg,**kw):
        return self.ui.add_notebook(pos,*arg,**{
            **{
                'color':'#202020','fg':'#cccccc','bg':'#202020',
                'activefg':'#cfcfcf','activebg':'#2d2d2d',
                'onfg':'#ffffff','onbg':'#282828',
                'scrollbg':'#2e2e2e','scrollcolor':'#9f9f9f','scrollon':'#a0a0a0',
            },**kw})

    def add_ratingbar(self,pos,*arg,**kw):
        return self.ui.add_ratingbar(pos,*arg,**{
            **{
                'fg':'#cccccc','bg':'#202020',
                'onfg':'#4CC2FF','onbg':'#4CC2FF',
            },**kw})

    def add_radiobox(self,pos,*arg,**kw):
        return self.ui.add_radiobox(pos,*arg,**{
            **{
                'fontfg':'#ffffff','fg':'#939393','bg':'#2a2a2a',
                'activefg':'#959595','activebg':'#2a2a2a',
                'onfg':'#4CC2FF','onbg':'#000000',
            },**kw})

    def add_notecard(self,pos,*arg,**kw):
        return self.ui.add_notecard(pos,*arg,**{
            **{
                'tfg':'#ffffff','tbg':'#2b2b2b','fg':'#ffffff',
                'bg':'#272727','sep':'#1d1d1d',
            },**kw})

    def add_pivot(self,pos,*arg,**kw):
        return self.ui.add_pivot(pos,*arg,**{
            **{
                'fg':'#a6a6a6','bg':'',
                'activefg':'#ffffff','activecolor':'#4CC2FF',
            },**kw})

    def add_button2(self,pos,*arg,**kw):
        return self.ui.add_button2(pos,*arg,**{
            **{
                'fg':'#ffffff','bg':'#2d2d2d',
                'activefg':'#ffffff','activebg':'#323232',
                'line':'#303030','linew':1,'activeline':'#202020',
                'onfg':'#cecece','onbg':'#272727','online':'#303030',
            },**kw})
    
    def add_accentbutton(self,pos,*arg,**kw):
        return self.ui.add_accentbutton(pos,*arg,**{
            **{
                'fg':'#000000','bg':'#4CC2FF',
                'activefg':'#000000','activebg':'#47B1E8',
                'line':'#5AC7FF','linew':1,'activeline':'#2B80CA',
                'onfg':'#215069','onbg':'#42A1D2','online':'#42A1D2',
            },**kw})
    
    def add_warningbutton(self,pos,*arg,**kw):
        return self.ui.add_warningbutton(pos,*arg,**{
            **{
                'fg':'#000000','bg':'#FF99A4',
                'activefg':'#000000','activebg':"#F57F90",
                'line':"#FC8290",'linew':1,'activeline':"#BD233A",
                'onfg':"#751F2C",'onbg':"#E96F81",'online':"#D83C5E",
            },**kw})
    
    def add_toolbutton(self,pos,*arg,**kw):
        return self.ui.add_toolbutton(pos,*arg,**{
            **{
                'fg':'#FFFFFF','bg':'#202020',
                'activefg':'#FFFFFF','activebg':'#2D2D2D',
                'line':'#202020','linew':1,'activeline':'#2D2D2D',
                'onfg':'#CECECE','onbg':'#292929','online':'#292929',
            },**kw})

    def add_expander(self,pos,*arg,**kw):
        return self.ui.add_expander(pos,*arg,**{
            **{
                'tfg':'#ffffff','tbg':'#2b2b2b',
                'bg':'#272727','sep':'#1d1d1d',
                'buttonfg':'#ffffff','buttonbg':'#2b2b2b','buttonline':'#2b2b2b',
                'activefg':'#ffffff','activebg':'#373737','activeline':'#373737',
                'onfg':'#ffffff','onbg':'#333333','online':'#333333',
            },**kw})

    def add_waitframe(self,pos,*arg,**kw):
        return self.ui.add_waitframe(pos,*arg,**{
            **{
                'fg':'#2d2d2d','bg':'#323232',
            },**kw})

    def add_listview(self,pos,*arg,**kw):
        return self.ui.add_listview(pos,*arg,**{
            **{
                'bg':'#202020','activebg':'#2d2d2d','oncolor':'#4CC2FF',
                'scrobg':'#2e2e2e','scroc':'#9a9a9a','scrooc':'#9f9f9f',
            },**kw})

    def add_treeview(self,pos,*arg,**kw):
        return self.ui.add_treeview(pos,*arg,**{
            **{
                'fg':'#ffffff','bg':'#202020',
                'onfg':'#ffffff','onbg':'#2d2d2d',
                'oncolor':'#4CC2FF','signcolor':'#9f9f9f',
            },**kw})

    def add_togglebutton(self,pos,*arg,**kw):
        return self.ui.add_togglebutton(pos,*arg,**{
            **{
                'fg':'#ffffff','bg':'#2d2d2d','line':"#303030",
                'activefg':'#000000','activebg':'#4CC2FF','activeline':'#5AC7FF',
            },**kw})
    
    def add_swipecontrol(self,pos,*arg,**kw):
        # return self.ui.add_swipecontrol(pos,*arg,**{
        #     **{
        #         'fg':'#ffffff','bg':'#202020','line':'#2d2d2d',
        #         'data':{'left':({'text':'✔️\nok','fg':'#000000','bg':'#a9a9a9','command':print},),
        #         'right':({'text':'❌\nclose'},)},
        #     },**kw})
        ...

    def add_picker(self,pos,*arg,**kw):
        return self.ui.add_picker(pos,*arg,**{
            **{
                'fg':'#cfcfcf','bg':'#2d2d2d','outline':'#3c3c3c',
                'activefg':'#ffffff','activebg':'#323232',
                'onfg':'#000000','onbg':'#4CC2FF',
                'buttonfg':'#ffffff','buttonbg':'#2d2d2d',
                'buttonactivefg':'#ffffff','buttonactivebg':'#383838',
                'buttononfg':'#ffffff','buttononbg':'#343434',
            },**kw})
    
    def add_menubutton(self,pos,*arg,**kw):
        return self.ui.add_menubutton(pos,*arg,**{
            **{
                'fg':"#ffffff",'bg':'#2d2d2d','line':'#3d3d3d',
                'activefg':'#ffffff','activebg':'#323232','activeline':'#202020',
                'onfg':'#cecece','onbg':'#272727','online':'#303030',
                'menuonfg':'#ffffff','menuonbg':'#383838','menuonline':'#383838',
            },**kw})
    
    def add_barbutton(self,pos,*arg,**kw):
        return self.ui.add_barbutton(pos,*arg,**{
            **{
                'fg':'#ffffff','bg':'#202020',
                'line':'#202020',
                'activefg':'#cecece','activebg':'#2d2d2d',
                'onfg':'#cecece','onbg':'#292929',
                'online':'#292929','sepcolor':'#e5e5e5',
            },**kw})

    def add_flyout(self,fid,*arg,**kw):
        return self.ui.add_flyout(fid,*arg,**{
            **{
                'line':'#b0b0b0','bg':'#2c2c2c',
            },**kw})

    def add_back(self,pos,*arg,**kw):
        return self.ui.add_back(pos,*arg,**{
            **{
                'fg':'#202020','bg':'#202020',
            },**kw})
    
    def add_breadcrumb(self,pos,*arg,**kw):
        return self.ui.add_breadcrumb(pos,*arg,**{
            **{
                'fg':'#ffffff','bg':'#202020','activefg':'#cccccc'
            },**kw})
    
    def add_segmentbutton(self,pos,*arg,**kw):
        return self.ui.add_segmentbutton(pos,*arg,**{
            **{
                'fg':'#FFFFFF','bg':'#1D1D1D',
                'activefg':'#FFFFFF','activebg':'#2A2A2A',
                'onbg':'#3A3A3A',
                'line':'#303030','sign':'#4CC2FF'
        },**kw})

    def add_navigation(self,pos,*arg,**kw):
        return self.ui.add_navigation(pos,*arg,**{
            **{
                'fg':'#FFFFFF','bg':'#202020',
                'activefg':'#FFFFFF','activebg':'#2C2C2C',
                'onfg':'#FFFFFF','onbg':'#2C2C2C','oncolor':'#4CC2FF'
            },**kw})



from window import win

def active(e):
    funcs[1]()
def disable(e):
    funcs[0]()

a=win()
u=a.u
_,_,funcs,_=u.add_link((5,5),'Smart-Space个人网站','https://smart-space.com.cn/')
u.add_button((5,40),'禁用链接',command=disable)
u.add_button((120,40),'恢复链接',command=active)
a.go()



from window import win

def change1(event):
    func.change_command(test1)
def test1(event):
    print('1')
def change2(event):
    func.change_command(test2)
def test2(event):
    print('2')

def disable(event):
    func2.disable()
def active(event):
    func2.active()

a=win()
u=a.u
u.add_button((10,5),'切换到函数1',command=change1)
u.add_button((150,5),'切换到函数2',command=change2)
u.add_button((250,5),'禁止按钮',command=disable)
u.add_button((350,5),'激活按钮',command=active)
_,_,func,_=u.add_button((5,100),'测试按钮',command=test1)
_,_,func2,_=u.add_button((100,100),'测试按钮2',command=test1)

a.go()



"""
TinUIXml简易设计编辑器

注意！！！这只是测试文件，不保证随TinUI正式版本进行维护，
仅作为对TinUIXml和BasicTinUI设计模式 show_location() 的测试。
本代码文件本身不具备xml文本和python代码专用编辑能力。

本文件可作为开发TinUIXml编辑器或通过绝对坐标设计TinUI界面的参考，
不附带TinUI开发能力，不要单纯依靠本程序开发TinUI及python应用。
"""
from window import *
from tkinter import Tk,Toplevel
from tkinter.ttk import Button
from tkinter.scrolledtext import ScrolledText
from tkinter.simpledialog import askinteger
import re
import idlelib.colorizer as idc
import idlelib.percolator as idp


def show_location(state:bool=True,color='red',command=None):
    #在设计时反馈鼠标所在的绝对位置
    if not state:
        displayui.unbind('<Enter>')
        displayui.unbind('<Motion>')
        displayui.unbind('<Leave>')
        return
    displayui.locx=None
    displayui.locy=None
    def entercall(e):
        bbox=displayui.bbox('all')
        width=displayui.winfo_width()
        height=displayui.winfo_height()
        if bbox==None:
            _width=0
            _height=0
        else:
            _width=bbox[2]-bbox[0]
            _height=bbox[3]-bbox[1]
        if _width>width:
            width=_width
        if _height>height:
            height=_height
        displayui.loclx=displayui.create_line((0,0,width,0),width=2,dash=(5,5),fill=color)
        displayui.locly=displayui.create_line((0,0,0,height),width=2,dash=(5,5),fill=color)
    def motioncall(e):#鼠标在界面中滑动
        if displayui.loclx!=None:
            x=displayui.canvasx(e.x)
            y=displayui.canvasy(e.y)
            displayui.moveto(displayui.loclx,0,y)
            displayui.moveto(displayui.locly,x,0)
            if command!=None:command(x,y)
    def leavecall(e):
        displayui.delete(displayui.loclx)
        displayui.delete(displayui.locly)
        displayui.loclx=None
        displayui.locly=None
    displayui.bind('<Enter>',entercall)
    displayui.bind('<Motion>',motioncall)
    displayui.bind('<Leave>',leavecall)

loclines=False#坐标十字线是否存在
def if_location(e):#是否显示坐标十字线
    global loclines
    loclines=e
    if loclines:
        tinui.itemconfig(loctext,state='normal')
        show_location(command=getloc)
    else:
        tinui.itemconfig(loctext,text='x:? y:?')
        show_location(False)

def getloc(x,y):
    tinui.itemconfig(loctext,text=f'x:{x} y:{y}')


def inxml(e):#注入xml界面
    displayui.delete('all')
    duixml.clean()
    xml=text.get(1.0,'end')
    duixml.funcs=dict()
    duixml.datas=dict()
    duixml.tags=dict()
    result=re.findall(r"self\.(\w+)\[(['\"])([^\"]*)\2\]",xml)
    for i in result:
        if i[0]=='funcs':
            duixml.funcs[i[2]]=None
        elif i[0]=='datas':
            duixml.datas[i[2]]=(None,None)
    duixml.loadxml(xml)
    rescroll()
    reset_marks()

def write(text):
    textbox.insert('end',text)
def pycode(e):#获取IDO代码
    inxml(e)
    pycodew.deiconify()
    textbox.configure(state='normal')
    textbox.delete(1.0,'end')
    uixml=text.get(1.0,'end')
    result=re.findall(r"self\.(.*?)\[.(.*?).\]",uixml,re.M|re.S)
    textbox.result=result
    write('''# u = TinUI(root)
# x = TinUIXml(u)
# xml=file.read()

''')
    write('''# in
''')
    for i in result:
        if i[0]=='funcs':
            write(f'x.funcs["{i[1]}"] = {i[1]}\n')
        elif i[0]=='datas':
            write(f'x.datas["{i[1]}"] = {i[1]}\n')
    write('''
# during
x.loadxml(xml)

# out
''')
    for tag in duixml.tags.keys():
        write(f'{tag} = x.tags["{tag}"]\n')
    textbox.configure(state='disabled')

def copy_pycode(e):#复制
    textbox.clipboard_clear()
    copyText = textbox.get(1.0,'end')
    textbox.clipboard_append(copyText)
def highlight(e):#标注funcs,datas等重点
    havefunc,havedata=False,False
    textbox.configure(state='normal')
    write('\n#TinUIXml导入重点：\n')
    for i in textbox.result:
        if i[0]=='funcs':
            if havefunc==False:
                havefunc=True
                write('#函数/方法(funcs)：\n')
            write(f'#  {i[1]}(...)\n')
        elif i[0]=='datas':
            if havedata==False:
                havedata=True
                write('#数据(datas)：\n')
            write(f'#  {i[1]}=...\n')
    havetag=False
    for tag in duixml.tags.keys():
        if havetag==False:
            havetag=True
            write('\n#TinUIXml导出重点：\n')
        write(f'#  {tag}\n')
    textbox.configure(state='disabled')

now_mark=None#mark_index
def open_markw(e):
    markw.deiconify()
def del_mark(e):#删除选定标记点
    global now_mark
    if now_mark==None:
        return
    listbox.delete(now_mark)
    displayui.delete(mark_points[now_mark][1])
    del mark_points[now_mark]
    now_mark=None
def add_mark(e):#手动添加标记点
    x=askinteger('手动添加标记点','横坐标：',parent=markw)
    if x==None:
        return
    y=askinteger('手动添加标记点','纵坐标：',parent=markw)
    if y==None:
        return
    __set_mark(x,y)
def sel_mark(name):#选定标记点
    global now_mark
    if name.index==0:
        now_mark=None
    else:
        if now_mark!=None:
            displayui.itemconfigure(mark_points[now_mark][1],outline='black',fill="black")
        now_mark=name.index
        displayui.itemconfigure(mark_points[now_mark][1],outline='red',fill='red')

def __set_mark(x,y):
    mark=displayui.create_oval((x,y,x+3,y+3),outline='black',fill="black")
    mark_points.append(((x,y),mark))
    listbox.add(f'({x} , {y})')
def set_mark(e):#绘制标记点
    __set_mark(e.x,e.y)
def reset_marks():#重新绘制标记点
    if len(mark_points)==0:
        return
    index=1
    for i in mark_points[1:]:
        mark=displayui.create_oval((i[0][0],i[0][1],i[0][0]+3,i[0][1]+3),outline='black',fill="black")
        mark_points[index]=(i[0],mark)
        index+=1


root=Tk()


root.geometry('1200x700')
root.title('TinUIXml设计测试')

initial_xml='''<!--TinUIXml编辑-->
<tinui>
<line>
    <line>
        <title text='back'></title>
    </line>
    <line>
        <paragraph text='TinUI的背景控件，用来呈现不同控件组的层次。' width='580'></paragraph>
    </line>
    <line>
    <paragraph text=' ' width='580'></paragraph>
    </line>
    <line anchor='w'>
    <paragraph text='下方就是back'>p1</paragraph>
    <button2 text='本按钮与左边文字构成控件组' command='self.funcs["show_msg"]'>b1</button2>
    <back uids='("b1","p1")' bg='yellow' fg='yellow'></back>
    </line>
</line>
</tinui>'''
tinui=BasicTinUI(root,bg='#f3f3f3')
tinui.pack(expand=True,fill='both')

textids=tinui.add_textbox((0,0),400,700,font='微软雅黑 13',scrollbar=True)
text=textids[0]
textid=textids[-1]
text.insert(1.0,initial_xml)

rp=ExpandPanel(tinui)
hp=HorizonPanel(tinui,spacing=5)
rp.set_child(hp)

vp=VerticalPanel(tinui)
hp.add_child(vp,400)
ep1=ExpandPanel(tinui)
vp.add_child(ep1,700,weight=1)
ep1.set_child(textid)

ep=ExpandPanel(tinui)
hp.add_child(ep,weight=1)

vp2=VerticalPanel(tinui)
ep.set_child(vp2)

top=HorizonPanel(tinui,spacing=5)
vp2.add_child(top,50)
btn=tinui.add_button2((0,0),text='导入xml',command=inxml,anchor='w')[-1]
top.add_child(btn,70)
btn=tinui.add_button2((0,0),text='python代码',command=pycode,anchor='w')[-1]
top.add_child(btn,100)
ctk=tinui.add_checkbutton((0,0),text='启用十字线定位',command=if_location,anchor='w')[-1]
top.add_child(ctk,150)
loctext=tinui.add_paragraph((0,0),text='x: y:',anchor='w')
top.add_child(loctext)

top.add_child(ExpandPanel(tinui),weight=1)

btn=tinui.add_button2((0,0),text='打开标记点管理窗口',command=open_markw,anchor='e')[-1]
top.add_child(btn,300)

uiep=ExpandPanel(tinui)
vp2.add_child(uiep,weight=1)

displayui,rescroll,duixml,uiid=tinui.add_ui((0,0),width=870,height=630,scrollbar='True',region='auto')
uiep.set_child(uiid)

def update(e):
    rp.update_layout(5,5,e.width-5,e.height-5)
tinui.bind('<Configure>',update)


#Python代码弹窗窗口
pycodew=Toplevel()
pycodew.title("Python代码")
# 设置窗口大小
window_width = 520
window_height = 550
# 获取屏幕大小
screen_width = pycodew.winfo_screenwidth()
screen_height = pycodew.winfo_screenheight()
# 计算窗口居中的x和y坐标
x_coordinate = int((screen_width/2) - (window_width/2))
y_coordinate = int((screen_height/2) - (window_height/2))
# 设置窗口的位置和大小，并禁止改变尺寸
pycodew.geometry("{}x{}+{}+{}".format(window_width, window_height,
                                      x_coordinate, y_coordinate))
pycodew.resizable(width=False, height=False)  # 禁止改变窗口大小
pctinui=BasicTinUI(pycodew)
pctinui.pack(fill='both',expand=True)
pycodew.protocol("WM_DELETE_WINDOW", lambda: pycodew.withdraw())  # 忽略关闭窗口的协议
pycodew.withdraw()
pctinuix=TinUIXml(pctinui)

xmlf=open(os.path.dirname(__file__)+r'\xmltestpage\pytest.xml','r',encoding='utf-8')
xml=xmlf.read()
xmlf.close()
pctinuix.environment(globals())
pctinuix.loadxml(xml)
textbox=pctinuix.tags['textbox'][0]
idc.color_config(textbox)
p = idp.Percolator(textbox)
d = idc.ColorDelegator()
p.insertfilter(d)


#==========
#标记点管理页面
markw = Toplevel(root)
markw.title("标记点管理")
markw.geometry("400x600")
markw.resizable(width=False, height=False)  # 禁止改变窗口大小
markw.protocol("WM_DELETE_WINDOW", lambda: markw.withdraw())  # 忽略关闭窗口的协议
markui=BasicTinUI(markw)
markui.pack(fill='both',expand=True)
markw.withdraw()
markuix=TinUIXml(markui)

xmlf=open(os.path.dirname(__file__)+r'\xmltestpage\marks.xml','r',encoding='utf-8')
xml=xmlf.read()
xmlf.close()
markuix.environment(globals())
markuix.loadxml(xml)
listbox=markuix.tags['listbox'][-2]
mark_points=[((None,None),None)]#与listbox内列表同步更新，[((x,y), point_uid), ...]
displayui.bind('<Button-3>',set_mark)

if __name__=='__main__':
    root.mainloop()



"""
TinUI风格对话框
"""
from tkinter import Tk, Toplevel

try:
    from .TinUI import BasicTinUI, TinUIXml
except:
    from TinUI import BasicTinUI, TinUIXml



class Dialog(Toplevel):
    """
    TinUI对话框基础类
    """

    def __init__(self,master,dialogtype='normal',theme='light',**options):
        """
        theme: 'dark' or 'light'
        """
        super().__init__(master,**options)
        self.withdraw()

        if master:
            self.transient(self.master)
        
        self.tinui=BasicTinUI(self)
        self.tinui.pack(fill='both',expand=True)

        self.type=dialogtype#对话框类型

        if theme=='light':
            self.background = '#f3f3f3'
            self.fg = '#000000'
            self.barback = '#f3f3f3'
            self.selback = '#1a1a1a'
            self.buttonargs = {
                'fg':'#1b1b1b',
                'bg':'#fbfbfb',
                'line':'#cccccc',
                'activefg':'#1a1a1a',
                'activebg':'#f6f6f6',
                'activeline':'#cccccc',
                'onfg':'#5d5d5d',
                'onbg':'#f5f5f5',
                'online':'#e5e5e5'
            }
            self.entryargs = {
                'fg':'#1b1b1b',
                'bg':'#fbfbfb',
                'activefg':'#1a1a1a',
                'activebg':'#f6f6f6',
                'onfg':'#000000',
                'onbg':'#ffffff',
                'line':'#e5e5e5',
                'activeline':'#e5e5e5',
                'insert':'#000000',
                'outline':'#868686',
                'onoutline':'#0067C0',
            }
            self.listargs = {
                'fg':'#1a1a1a',
                'bg':'#f2f2f2',
                'activefg':'#191919',
                'activebg':'#e9e9e9',
                'onfg':'#191919',
                'onbg':'#ececec',
                'sel':'#91C1E6',
                'scrollbg':'#f9f9f9',
                'scrollcolor':'#8d8d8d',
                'scrollon':'#8a8a8a'
            }
            self.infocolor = '#5969e0'
            self.infoback = '#ffffff'
            self.successcolor = '#0f7b0f'
            self.successback = '#dff6dd'
            self.warningcolor = '#9d5d00'
            self.warningback = '#fff4ce'
            self.errorcolor = '#c42b1c'
            self.errorback = '#fde7e9'
            self.questioncolor = '#5969e0'
            self.questionback = '#ffffff'
        elif theme=='dark':
            self.background = '#202020'
            self.fg = '#ffffff'
            self.barback = '#202020'
            self.selback = '#ffffff'
            self.buttonargs = {
                'fg':'#ffffff',
                'bg':'#2d2d2d',
                'line':'#303030',
                'activefg':'#ffffff',
                'activebg':'#323232',
                'activeline':'#202020',
                'onfg':'#cecece',
                'onbg':'#272727',
                'online':'#303030'
            }
            self.entryargs = {
                'fg':'#cfcfcf',
                'bg':'#2d2d2d',
                'activefg':'#ffffff',
                'activebg':'#323232',
                'onfg':'#ffffff',
                'onbg':'#1f1f1f',
                'line':'#303030',
                'activeline':'#202020',
                'outline':'#9a9a9a',
                'onoutline':'#4CC2FF',
                'insert':'#e0e0e0',
            }
            self.listargs = {
                'bg':'#2d2d2d',
                'fg':'#ffffff',
                'activefg':'#ffffff',
                'activebg':'#373737',
                'onfg':'#ffffff',
                'onbg':'#333333',
                'sel':'#115990',
                'scrollbg':'#2e2e2e',
                'scrollcolor':'#9f9f9f',
                'scrollon':'#a0a0a0'
            }
            self.infocolor = '#4cc2ff'
            self.infoback = '#2c2c2c'
            self.successcolor = '#6ccb5f'
            self.successback = '#393d1b'
            self.warningcolor = '#fce100'
            self.warningback = '#433519'
            self.errorcolor = '#ff99a4'
            self.errorback = '#442726'
            self.questioncolor = '#4cc2ff'
            self.questionback = '#2c2c2c'

        self.resizable(False,False)
        self.tk.call('wm', 'iconbitmap', self._w, '-default', '')
    
    def _endy(self):
        #最低位置
        bbox=self.tinui.bbox('all')
        if bbox:
            return bbox[3]
        else:
            return 0
    
    def initial_msg(self,title,content,yestext='OK',notext='Cancel'):
        """
        初始化对话框-消息类
        """
        YES=yestext
        NO=notext

        self.title(title)
        self.protocol('WM_DELETE_WINDOW',lambda:self.return_msg(None))
        
        # contenty=5
        if self.type=='normal':
            self.tinui['bg']=self.background
        elif self.type=='info':
            self.tinui['bg']=self.infoback
            icon_uid=self.tinui.add_paragraph((5,5),text='\uE946',fg=self.infocolor,font='{Segoe Fluent Icons} 14',anchor='w')
            self.tinui.addtag_withtag('content',icon_uid)
        elif self.type=='success':
            self.tinui['bg']=self.successback
            icon_uid=self.tinui.add_paragraph((5,5),text='\uE73E',fg=self.successcolor,font='{Segoe Fluent Icons} 14',anchor='w')
            self.tinui.addtag_withtag('content',icon_uid)
        elif self.type=='warning':
            self.tinui['bg']=self.warningback
            icon_uid=self.tinui.add_paragraph((5,5),text='\uE7BA',fg=self.warningcolor,font='{Segoe Fluent Icons} 14',anchor='w')
            self.tinui.addtag_withtag('content',icon_uid)
        elif self.type=='error':
            self.tinui['bg']=self.errorback
            icon_uid=self.tinui.add_paragraph((5,5),text='\uEA39',fg=self.errorcolor,font='{Segoe Fluent Icons} 14',anchor='w')
            self.tinui.addtag_withtag('content',icon_uid)
        elif self.type=='question':
            self.tinui['bg']=self.questionback
            icon_uid1=self.tinui.add_paragraph((5,5),text='\uEA3A',fg=self.questioncolor,font='{Segoe Fluent Icons} 14',anchor='w')
            icon_uid2=self.tinui.add_paragraph((5,5),text='\uF142',fg=self.questioncolor,font='{Segoe Fluent Icons} 14',anchor='w')
            self.tinui.addtag_withtag('content',icon_uid1)
            self.tinui.addtag_withtag('content',icon_uid2)

        content_uid=self.tinui.add_paragraph((35,5),text=content,fg=self.fg,anchor='w')
        self.tinui.addtag_withtag('content',content_uid)
        content_bbox=self.tinui.bbox('content')
        btn_width=(content_bbox[2]-content_bbox[0])/2
        button_width=btn_width-10 if btn_width>110 else 100
        button_endy=self._endy()+15
        yesbutton_uid=self.tinui.add_button2(((content_bbox[0]+content_bbox[2])/2-5,button_endy),text=YES,minwidth=button_width,command=lambda e:self.return_msg(True),anchor='ne',**self.buttonargs)[-1]
        yb_coords = self.tinui.coords(yesbutton_uid)
        nobutton_uid=self.tinui.add_button2(((content_bbox[0]+content_bbox[2])/2+5,button_endy),text=NO,minwidth=button_width,command=lambda e:self.return_msg(False),anchor='nw',**self.buttonargs)[-1]
        nb_coords = self.tinui.coords(nobutton_uid)
        self.tinui.add_back((),(yesbutton_uid,nobutton_uid),bg=self.barback,fg=self.barback,linew=8)

        def return_focus(event):
            if now_focus == 'yes':
                self.return_msg(True)
            else:
                self.return_msg(False)
        def focus_left(event):
            nonlocal now_focus
            if now_focus == 'yes':
                return
            now_focus = 'yes'
            self.tinui.coords(focus_button, yb_coords)
        def focus_right(event):
            nonlocal now_focus
            if now_focus == 'no':
                return
            now_focus = 'no'
            self.tinui.coords(focus_button, nb_coords)

        focus_button = self.tinui.create_polygon(yb_coords, width=11, fill=self.barback, outline=self.selback)
        self.tinui.lower(focus_button, yesbutton_uid)
        now_focus = 'yes'
        self.bind('<Return>', return_focus)
        self.bind('<space>', return_focus)
        self.bind('<Left>', focus_left)
        self.bind('<Right>', focus_right)

        self.entryw = None
        return self.load_window()
    
    def return_msg(self,val):
        #返回消息
        self.result=val
        self.destroy()
        self.master.focus_set()

    def initial_input(self,title,content,text,yestext='OK',notext='Cancel'):
        """
        初始化对话框-输入类
        """
        YES=yestext
        NO=notext

        self.tinui['bg']=self.background
        
        self.title(title)
        self.protocol('WM_DELETE_WINDOW',lambda:self.return_input(None))
        
        self.tinui.add_paragraph((5,5),text=content,fg=self.fg)
        content_bbox=self.tinui.bbox('all')
        entry_width=content_bbox[2]-content_bbox[0]
        width=entry_width if entry_width>200 else 200
        self.entryw, self.entry=self.tinui.add_entry((5,self._endy()+5),width=width,**self.entryargs)[:-1]# tinui entry widget, funcs
        self.entry.insert(0,str(text))
        bbox=self.tinui.bbox('all')
        btn_width=(bbox[2]-bbox[0])/2
        button_width=btn_width-10 if btn_width>110 else 100
        button_endy=self._endy()+15
        yesbutton_uid=self.tinui.add_button2(((bbox[0]+bbox[2])/2-5,button_endy),text=YES,minwidth=button_width,command=lambda e:self.return_input(self.entry.get()),anchor='ne',**self.buttonargs)[-1]
        yb_coords = self.tinui.coords(yesbutton_uid)
        nobutton_uid=self.tinui.add_button2(((bbox[0]+bbox[2])/2+5,button_endy),text=NO,minwidth=button_width,command=lambda e:self.return_input(None),anchor='nw',**self.buttonargs)[-1]
        self.tinui.add_back((),(yesbutton_uid,nobutton_uid),bg=self.barback,fg=self.barback,linew=8)

        focus_button = self.tinui.create_polygon(yb_coords, width=11, fill=self.barback, outline=self.selback)
        self.tinui.lower(focus_button, yesbutton_uid)
        self.bind('<Return>', lambda e:self.return_input(self.entry.get()))

        return self.load_window()

    def return_input(self,val):
        #返回输入内容
        self.result=val
        if self.result==None:
            pass
        elif self.type=='string':
            pass
        elif self.type=='integer':
            try:
                self.result=int(self.result)
            except:
                return
        elif self.type=='float':
            try:
                self.result=float(self.result)
            except:
                return
        self.destroy()
        self.master.focus_set()
    
    def initial_choice(self,title,content,choices,yestext='OK',notext='Cancel'):
        """
        初始化对话框-选择类
        """
        YES=yestext
        NO=notext
        self.result=None

        self.tinui['bg']=self.background

        self.title(title)
        self.protocol('WM_DELETE_WINDOW',lambda:self.return_choice(None))

        self.tinui.add_paragraph((5,5),text=content,fg=self.fg)
        content_bbox=self.tinui.bbox('all')
        width = max(content_bbox[2]-content_bbox[0], 300)

        if self.type=='listbox':
            self.tinui.add_listbox((5,self._endy()+5),width=width,height=300,data=choices,command=self.return_choice,**self.listargs)

        bbox=self.tinui.bbox('all')
        btn_width=(bbox[2]-bbox[0])/2
        button_width=btn_width-10 if btn_width>110 else 100
        button_endy=self._endy()+15
        yesbutton_uid=self.tinui.add_button2(((bbox[0]+bbox[2])/2-5,button_endy),text=YES,minwidth=button_width,command=lambda e:self.return_choice(True),anchor='ne',**self.buttonargs)[-1]
        yb_coords = self.tinui.coords(yesbutton_uid)
        nobutton_uid=self.tinui.add_button2(((bbox[0]+bbox[2])/2+5,button_endy),text=NO,minwidth=button_width,command=lambda e:self.return_choice(None),anchor='nw',**self.buttonargs)[-1]
        nb_coords = self.tinui.coords(nobutton_uid)
        self.tinui.add_back((),(yesbutton_uid,nobutton_uid),bg=self.background,fg=self.background,linew=8)

        def return_focus(event):
            if now_focus == 'yes':
                self.return_choice(True)
            else:
                self.return_choice(None)
        def focus_left(event):
            nonlocal now_focus
            if now_focus == 'yes':
                return
            now_focus = 'yes'
            self.tinui.coords(focus_button, yb_coords)
        def focus_right(event):
            nonlocal now_focus
            if now_focus == 'no':
                return
            now_focus = 'no'
            self.tinui.coords(focus_button, nb_coords)

        focus_button = self.tinui.create_polygon(yb_coords, width=11, fill=self.barback, outline=self.selback)
        self.tinui.lower(focus_button, yesbutton_uid)
        now_focus = 'yes'
        self.bind('<Return>', return_focus)
        self.bind('<space>', return_focus)
        self.bind('<Left>', focus_left)
        self.bind('<Right>', focus_right)

        self.entryw = None
        return self.load_window()
    
    def return_choice(self,val):
        #返回选择内容
        if val==True:#YES
            if self.result==None:
                return
            self.destroy()
            self.master.focus_set()
        elif val==None:#NO
            self.result=None
            self.destroy()
            self.master.focus_set()
        else:#输入值
            self.result=val

    def load_window(self):
        #获取窗口内所有控件的bbox，窗口居中布局
        bboxall=self.tinui.bbox('all')
        w,h=bboxall[2]-bboxall[0],bboxall[3]-bboxall[1]
        screenw=self.winfo_screenwidth()
        screenh=self.winfo_screenheight()
        x,y=(screenw-w)/2,(screenh-h)/2
        self.geometry(f'{w}x{h}+{int(x)}+{int(y)-10}')
        self.deiconify()

        self.tinui.config(scrollregion=bboxall)

        self.focus_set()
        if self.entryw:
            self.entryw.focus_set()
        self.wait_visibility()
        self.grab_set()
        self.wait_window(self)

        return self.result
    
    def initial_xml_load(self,title,xml,funcdict:dict={},data:dict={},yestext='OK',notext='Cancel',yescallback=None,nocallback=None,nonecallback=None,tinuitheme=None):
        """
        自定义XML内容
        """
        YES=yestext
        NO=notext
        self.tinui['bg'] = self.background

        if tinuitheme:
            self.tinuitheme = tinuitheme(self.tinui)
            self.tinuixml = TinUIXml(self.tinuitheme)
        else:
            self.tinuixml = TinUIXml(self.tinui)
        self.tinuixml.funcs.update(funcdict)
        self.tinuixml.datas.update(data)

        self.title(title)
        self.protocol('WM_DELETE_WINDOW',lambda:(nonecallback() ,self.return_msg(None)))

        self.tinuixml.loadxml(xml)

        content_bbox=self.tinui.bbox('all')
        btn_width=(content_bbox[2]-content_bbox[0])/2
        button_width=btn_width-10 if btn_width>110 else 100
        button_endy=self._endy()+15
        yesbutton_uid=self.tinui.add_button2(((content_bbox[0]+content_bbox[2])/2-5,button_endy),text=YES,minwidth=button_width,command=lambda e:(yescallback() ,self.return_msg(True)),anchor='ne',**self.buttonargs)[-1]
        yb_coords = self.tinui.coords(yesbutton_uid)
        nobutton_uid=self.tinui.add_button2(((content_bbox[0]+content_bbox[2])/2+5,button_endy),text=NO,minwidth=button_width,command=lambda e:(nocallback() ,self.return_msg(False)),anchor='nw',**self.buttonargs)[-1]
        nb_coords = self.tinui.coords(nobutton_uid)
        self.tinui.add_back((),(yesbutton_uid,nobutton_uid),bg=self.barback,fg=self.barback,linew=8)

        def return_focus(event):
            if now_focus == 'yes':
                yescallback()
                self.return_msg(True)
            else:
                nocallback()
                self.return_msg(False)
        def focus_left(event):
            nonlocal now_focus
            if now_focus == 'yes':
                return
            now_focus = 'yes'
            self.tinui.coords(focus_button, yb_coords)
        def focus_right(event):
            nonlocal now_focus
            if now_focus == 'no':
                return
            now_focus = 'no'
            self.tinui.coords(focus_button, nb_coords)

        focus_button = self.tinui.create_polygon(yb_coords, width=11, fill=self.barback, outline=self.selback)
        self.tinui.lower(focus_button, yesbutton_uid)
        now_focus = 'yes'
        self.bind('<Return>', return_focus)
        self.bind('<space>', return_focus)
        self.bind('<Left>', focus_left)
        self.bind('<Right>', focus_right)

        self.entryw = None

    def initial_xml_init(self):
        return self.load_window()



def show_msg(master,title,content,theme='light'):
    """
    显示消息对话框
    """
    dialog=Dialog(master,'normal',theme)
    return dialog.initial_msg(title,content)

def show_info(master,title,content,yestext='OK',notext='Cancel',theme='light'):
    """
    显示信息对话框
    """
    dialog=Dialog(master,'info',theme)
    return dialog.initial_msg(title,content,yestext,notext)
def show_success(master,title,content,yestext='OK',notext='Cancel',theme='light'):
    """
    显示成功对话框
    """
    dialog=Dialog(master,'success',theme)
    return dialog.initial_msg(title,content,yestext,notext)

def show_warning(master,title,content,yestext='OK',notext='Cancel',theme='light'):
    """
    显示警告对话框
    """
    dialog=Dialog(master,'warning',theme)
    return dialog.initial_msg(title,content,yestext,notext)

def show_error(master,title,content,yestext='OK',notext='Cancel',theme='light'):
    """
    显示错误对话框
    """
    dialog=Dialog(master,'error',theme)
    return dialog.initial_msg(title,content,yestext,notext)

def show_question(master,title,content,yestext='YES',notext='NO',theme='light'):
    """
    显示询问对话框
    """
    dialog=Dialog(master,'question',theme)
    return dialog.initial_msg(title,content,yestext,notext)


def ask_string(master,title,content,text:str="",yestext='OK',notext='Cancel',theme='light'):
    """
    输入字符串对话框
    """
    dialog=Dialog(master,'string',theme)
    return dialog.initial_input(title,content,text,yestext,notext)

def ask_integer(master,title,content,text:int=0,yestext='OK',notext='Cancel',theme='light'):
    """
    输入整数对话框
    """
    dialog=Dialog(master,'integer',theme)
    return dialog.initial_input(title,content,text,yestext,notext)

def ask_float(master,title,content,text:float="0.0",yestext='OK',notext='Cancel',theme='light'):
    """
    输入浮点数对话框
    """
    dialog=Dialog(master,'float',theme)
    return dialog.initial_input(title,content,text,yestext,notext)

def ask_choice(master,title,content,choices,yestext='OK',notext='Cancel',theme='light'):
    """
    选择列表对话框
    """
    dialog=Dialog(master,'listbox',theme)
    return dialog.initial_choice(title,content,choices,yestext,notext)



# 此行（不含）以下代码不受GPLv3、LGPLv3许可证的限制，可以自由使用、修改、分发等。
if __name__=='__main__':
    root=Tk()
    root.iconbitmap('LOGO.ico')
    a=show_msg(root,'test','hello world!',theme='dark')
    print(a)
    show_info(root,'test','show information\nhello world!',theme='dark')
    show_success(root,'test','Success!\nhello world! hello world! hello world! hello world!',theme='dark')
    show_warning(root,'test','this is a warning\nhello world!',theme='dark')
    show_error(root,'test','something is wrong\nhello world! hello world! hello world! hello world!')
    show_question(root,'test','Do you want to continue?')
    b=ask_string(root,'test','input something input something input something input something')
    print(b)
    ask_integer(root,'test','input integer')
    ask_float(root,'test','input float')
    c=ask_choice(root,'test','choose one',('a','b','c'),theme='dark')
    print(c)
    xml_data = 0
    def xml_func(e):
        global xml_data
        xml_data = 1
    func_dict = {'func':xml_func}
    d=Dialog(root,'xml')
    d.initial_xml_load('test xml dialog','<tinui><line>'
    '<button text="click to change data value" command="self.funcs[\'func\']"></button>'
    '</line></tinui>',
    funcdict=func_dict,yescallback=lambda:print('yes'),nocallback=lambda:print('no'),nonecallback=lambda:print('none'))
    dr=d.initial_xml_init()
    if dr:
        print(f'yes with {xml_data}')
    elif dr==False:
        print(f'no with {xml_data}')
    elif dr==None:
        print(f'none with {xml_data}')
    root.mainloop()


from window import win

def step1():
    for i in range(1,5):
        funcs.add(f'item{i}')
def step2():
    funcs.delete()
def step3():
    cid = funcs.get()[1]
    funcs.delete_to(cid)

def unwrap(cids):
    res=[]
    for i in cids:
        res.append(u.itemcget(i,'text'))
    u.itemconfig(t,text='\\'.join(res))

a=win()
u=a.u
funcs=u.add_breadcrumb((250,5),anchor='n',command=unwrap)[-2]

step=1
def nextstep(e):
    global step
    if step==1:
        step1()
        step=2
    elif step==2:
        step2()
        step=3
    elif step==3:
        step3()
        u.delete(b)
b=u.add_button((5,50),'下一步',command=nextstep)[-1]
t=u.add_paragraph((250,55),'HOME',anchor='n')
a.go()



import setuptools

with open("README.md", "r",encoding='utf-8') as f:
  long_description = f.read()

setuptools.setup(name='tinui',  # 包名
      version='6.14.0',  # 版本号
      description='使用tkinter.Canvas绘制现代UI组件',
      long_description=long_description,
      long_description_content_type='text/markdown',
      author='Smart-Space',
      author_email='smart-space@qq.com',
      url='https://github.com/Smart-Space/TinUI',
      python_requires='>=3.6',
      license='GPL License',
      packages=setuptools.find_packages(),
      include_package_data = True,
      classifiers=[
          'Intended Audience :: Developers',
          'Natural Language :: Chinese (Simplified)',
          'Programming Language :: Python',
          'Topic :: Software Development :: Libraries'
      ],
      )



from window import win
import sys
sys.path.append('..')
from TinUI import ExpandPanel, HorizonPanel

def callback(tag):
    if isinstance(tag, bool):
        u.event_generate("<Configure>", width=u.winfo_width(), height=u.winfo_height())
    else:
        textbox[0].delete(1.0, "end")
        textbox[0].insert(1.0, f'{tag}-view')

a=win()
u=a.u
u.config(bg='#FFFFFF')

root=ExpandPanel(u)
hp=HorizonPanel(u,spacing=5)
root.set_child(hp)

textbox=u.add_textbox((0,0))
naview=u.add_navigation((0,0),command=callback)[-1]
hp.add_child(naview)

expand=ExpandPanel(u,padding=(5,5,5,5))
hp.add_child(expand, weight=1)
expand.set_child(textbox[-1])

def update(e):
    root.update_layout(5,5,e.width-5,e.height-5)
u.bind('<Configure>',update)

a.go()



from window import win

def select(e):
    funcs[0](2)
def disable(e):
    funcs[1]()
def active(e):
    funcs[2]()

a=win()
u=a.u
u.add_button((2,5),'选定第三个值',command=select)
u.add_button((150,5),'禁用',command=disable)
u.add_button((220,5),'启用',command=active)
_,_,_,funcs,_=u.add_combobox((2,50),text='test combobox',height=180,content=('1','2','3','4','5'))

a.go()



from window import win

def additem(e):
    funcs.add('new item new item new item new item new item')
def deleteitem(e):
    funcs.delete(1)
def deleteall(e):
    funcs.clear()
def print__(name):
    print(name,name.index)

a=win()
u=a.u
_,_,funcs,_=u.add_listbox((5,5),data=(),command=print__)

u.add_button((5,300),'添加元素',command=additem)
#u.add_button((120,300),'删除第二个元素'+'\uE755',font='{Segoe MDL2} 12',command=deleteitem)
u.add_button((120,300),'删除第二个元素',command=deleteitem)
u.add_button((120,350),'清空元素',command=deleteall)
a.go()
